<?php

namespace Abit\Webhook\Block\Adminhtml\Hook\Edit;

use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Form\Generic;
use Magento\Framework\Data\FormFactory;
use Magento\Framework\Registry;
use Abit\Webhook\Model\Config\Source\Status;
use Abit\Webhook\Model\Config\Source\Method;
use Abit\Webhook\Model\Config\Source\HookType;

class Form extends Generic
{

    /**
     * @var Status
     */
    private $_status;
    /**
     * @var Method
     */
    private $_method;
    /**
     * @var HookType
     */
    private $_hooktype;

    public function __construct(
        Context $context,
        Registry $registry,
        FormFactory $formFactory,
        Status $status,
        Method $method,
        HookType $hookType,
        array $data = []
    ) {
        $this->_status = $status;
        $this->_method = $method;
        $this->_hooktype = $hookType;
        parent::__construct($context, $registry, $formFactory, $data);
    }

    protected function _prepareForm()
    {
        $model = $this->_coreRegistry->registry('hook_data');
        $form = $this->_formFactory->create(
            ['data' => [
                'id' => 'edit_form',
                'enctype' => 'multipart/form-data',
                'action' => $this->getData('action'),
                'method' => 'post'
            ]
            ]
        );

        $form->setHtmlIdPrefix('hook_');
        if ($model->getHookId()) {
            $fieldset = $form->addFieldset(
                'base_fieldset',
                ['legend' => __('Edit Web Hook'), 'class' => 'fieldset-wide']
            );
            $fieldset->addField('hook_id', 'hidden', ['name' => 'hook_id']);
        } else {
            $fieldset = $form->addFieldset(
                'base_fieldset',
                ['legend' => __('Add Web Hook'), 'class' => 'fieldset-wide']
            );
        }

        $fieldset->addField(
            'name',
            'text',
            [
                'name' => 'name',
                'label' => __('Name'),
                'id' => 'name',
                'title' => __('Name'),
                'class' => 'required-entry',
                'required' => true,
            ]
        );

        $fieldset->addField(
            'delivery_url',
            'text',
            [
                'name' => 'delivery_url',
                'label' => __('Delivery Url'),
                'id' => 'name',
                'title' => __('Delivery Url'),
                'class' => 'required-entry',
                'required' => true,
            ]
        );

        $fieldset->addField(
            'webhook_secret',
            'text',
            [
                'name' => 'webhook_secret',
                'label' => __('WebHook Secret'),
                'id' => 'webhook_secret',
                'title' => __('WebHook Secret'),
                'class' => 'webhook_secret',
            ]
        );

        $fieldset->addField(
            'status',
            'select',
            [
                'name' => 'status',
                'label' => __('Status'),
                'id' => 'status',
                'title' => __('Status'),
                'values' => $this->_status->toOptionArray(),
                'class' => 'status',
                'required' => true,
            ]
        );

        $fieldset->addField(
            'method',
            'select',
            [
                'name' => 'method',
                'label' => __('Method'),
                'id' => 'method',
                'title' => __('Method'),
                'values' => $this->_method->toOptionArray(),
                'class' => 'method',
            ]
        );

        $fieldset->addField(
            'webhook_topic',
            'select',
            [
                'name' => 'webhook_topic',
                'label' => __('WebHook Topic'),
                'id' => 'webhook_topic',
                'title' => __('WebHook Topic'),
                'values' => $this->_hooktype->toOptionArray(),
                'class' => 'webhook_topic',
                'required' => true,
            ]
        );


        $form->setValues($model->getData());
        $form->setUseContainer(true);
        $this->setForm($form);

        return parent::_prepareForm();
    }
}
