<?php

namespace Abit\Webhook\Block\Adminhtml\Logs\Edit;

use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Form\Generic;
use Magento\Framework\Data\FormFactory;
use Magento\Framework\Registry;
use Abit\Webhook\Model\Config\Source\Status;
use Abit\Webhook\Model\Config\Source\Method;
use Abit\Webhook\Model\Config\Source\HookType;

class Form extends Generic
{

    /**
     * @var Status
     */
    private $_status;
    /**
     * @var Method
     */
    private $_method;
    /**
     * @var HookType
     */
    private $_hooktype;

    public function __construct(
        Context $context,
        Registry $registry,
        FormFactory $formFactory,
        Status $status,
        Method $method,
        HookType $hookType,
        array $data = []
    ) {
        $this->_status = $status;
        $this->_method = $method;
        $this->_hooktype = $hookType;
        parent::__construct($context, $registry, $formFactory, $data);
    }

    protected function _prepareForm()
    {
        $model = $this->_coreRegistry->registry('hook_log_data');
        $form = $this->_formFactory->create(
            ['data' => [
                'id' => 'edit_form',
                'enctype' => 'multipart/form-data',
                'action' => $this->getData('action'),
                'method' => 'post'
            ]
            ]
        );

        $form->setHtmlIdPrefix('hook_');
        $form->setFieldNameSuffix('log');

        $fieldset = $form->addFieldset('base_fieldset', [
            'legend' => __('General Information'),
            'class'  => 'fieldset-wide'
        ]);


        $fieldset->addField(
            'hook_id',
            'text',
            [
                'name' => 'hook_id',
                'label' => __('Hook id'),
                'id' => 'hook_name',
                'title' => __('Hook Name'),
                'class' => 'required-entry',
                'required' => true,
                'readonly' => true,
            ]
        );

        $fieldset->addField(
            'delivery_url',
            'text',
            [
                'name' => 'delivery_url',
                'label' => __('Delivery Url'),
                'id' => 'delivery_url',
                'title' => __('Delivery Url'),
                'class' => 'required-entry',
                'required' => true,
            ]
        );

        $fieldset->addField(
            'response',
            'textarea',
            [
                'name' => 'response',
                'label' => __('Response'),
                'id' => 'response',
                'title' => __('Response'),
                'class' => 'response',
                'rows' => 5
            ]
        );

        $fieldset->addField(
            'status',
            'select',
            [
                'name' => 'status',
                'label' => __('Status'),
                'id' => 'status',
                'title' => __('Status'),
                'values' => $this->_status->toOptionArray(),
                'class' => 'status',
                'required' => true,
            ]
        );

        $fieldset->addField(
            'message',
            'textarea',
            [
                'name' => 'message',
                'label' => __('Message'),
                'id' => 'message',
                'title' => __('Message'),
                'class' => 'hook_message',
                'rows' => 5
            ]
        );

        $fieldset->addField(
            'body',
            'textarea',
            [
                'name' => 'body',
                'label' => __('Body'),
                'id' => 'body',
                'title' => __('Body'),
                'class' => 'body',
                'rows' => 10
            ]
        );


        $form->setValues($model->getData());
        $form->setUseContainer(true);
        $this->setForm($form);

        return parent::_prepareForm();
    }
}
