<?php

namespace Abit\Webhook\Cron;

use Abit\Webhook\Helper\Data;
use Abit\Webhook\Model\HookFactory;
use Abit\Webhook\Model\HookLogFactory;
use Exception;
use Psr\Log\LoggerInterface;

class CronHook
{
    /**
     * @var HookLogFactory
     */
    private $_hookLogFactory;
    /**
     * @var Data
     */
    private $_helperData;
    /**
     * @var HookFactory
     */
    private $_hookFactory;
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param Data $data
     * @param HookFactory $hookFactory
     * @param HookLogFactory $hookLogFactory
     * @param LoggerInterface $logger
     */
    public function __construct(
        Data $data,
        HookFactory $hookFactory,
        HookLogFactory $hookLogFactory,
        LoggerInterface $logger
    ) {
        $this->_hookLogFactory = $hookLogFactory;
        $this->_hookFactory = $hookFactory;
        $this->_helperData = $data;
        $this->logger = $logger;
    }

    public function execute()
    {
        $hookLogCollection = $this->_hookLogFactory->create()->getCollection()
            ->addFieldToFilter('status', 0);

        foreach ($hookLogCollection as $hookLog) {
            $hook_id = $hookLog->getId();
            $obj = $hookLog->getBody();

            try {
                $result = $this->_helperData->send($obj, false, $hook_id);
            } catch (Exception $e) {
                $result = [
                    'success' => false,
                    'message' => $e->getMessage()
                ];
            }
            if ($result['success'] === true) {
                $this->logger->info('success');
            } else {
                $this->logger->info($result['message']);
            }
        }
    }
}
