<?php

namespace Abit\Webhook\Helper;

use Abit\Webhook\Model\HookFactory;
use Abit\Webhook\Model\HookLogFactory;
use Abit\Webhook\Model\ResourceModel\Hook\Collection;
use Exception;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\HTTP\Adapter\CurlFactory;
use Magento\Framework\ObjectManagerInterface;
use Zend_Http_Response;

class Data extends AbstractHelper
{
    /**
     * @var ObjectManagerInterface
     */
    private $_objectManager;
    /**
     * @var CurlFactory
     */
    private $_curlFactory;
    /**
     * @var HookFactory
     */
    private $_hookFactory;

    /**
     * @var HookLogFactory
     */
    private $_hookLogFactory;

    public function __construct(
        Context $context,
        ObjectManagerInterface $objectManager,
        HookFactory $hookFactory,
        HookLogFactory $hookLogFactory,
        CurlFactory $curlFactory
    ) {
        $this->_objectManager = $objectManager;
        $this->_curlFactory = $curlFactory;
        $this->_hookFactory = $hookFactory;
        $this->_hookLogFactory = $hookLogFactory;
        parent::__construct($context);
    }

    public function send($item, $log = false, $id_log = null)
    {
        /** @var Collection $hookCollection */
        $hookCollection = $this->_hookFactory->create()->getCollection()
            ->addFieldToFilter('status', 1);

        foreach ($hookCollection as $hook) {
            $hookLog = $this->_hookLogFactory->create();

            $url = $hook->getDeliveryUrl();

            if ($log == false && $id_log == null) {
                $data = [
                    'hook_id' => $hook->getId(),
                    'hook_name' => $hook->getName(),
                    'delivery_url' => $url,
                    'body' => $item
                ];

                $hookLog->addData($data);
            } else {
                $hookLog->setId($id_log);
                $hookLog->setDeliveryUrl($url);
            }

            try {
                $result = $this->sendHttpRequestData($hook, $item);
                $hookLog->setResponse(isset($result['response']) ? $result['response'] : '');
            } catch (Exception $e) {
                $result = [
                    'success' => false,
                    'message' => $e->getMessage()
                ];
            }

            if ($result['success'] === true) {
                $hookLog->setStatus(1);
                $hookLog->setMessage('success');
            } else {
                $hookLog->setStatus(0);
                $hookLog->setMessage($result['message']);
            }

            $hookLog->save();
        }
    }

    /**
     * @param $basic_auth
     * @return string
     */
    public function getBasicAuthorization($basic_auth)
    {
        return 'Basic ' . base64_encode($basic_auth);
    }

    public function sendHttpRequestData($hook, $item = false, $hookLog = false)
    {
        $url = $hookLog ? $hookLog->getDeliveryUrl() : $hook->getDeliveryUrl();
        $contentType = 'application/json';
        $method = $hook->getMethod();
        $hook_secret = $hook->getWebhookSecret();
        $basic_auth = $this->getBasicAuthorization($hook_secret);
        $body = $hookLog ? $hookLog->getBody() : $item;

        return $this->sendHttpRequest($contentType, $url, $method, $basic_auth, $body);
    }

    public function sendHttpRequest($contentType, $url, $method, $basic_auth, $body)
    {
        $headersConfig = [];
        $http_version = '1.1';
        $timeout = 15;
        $config = [
            'timeout' => $timeout
        ];

        if (!$method) {
            $method = 'GET';
        }

        if ($contentType) {
            $headersConfig[] = 'Content-Type: ' . $contentType;
        }
        if ($basic_auth) {
            $headersConfig[] = 'Authorization: ' . $basic_auth;
        }

        $curl = $this->_curlFactory->create();
        $curl->setConfig($config);
        $curl->write($method, $url, $http_version, $headersConfig, $body);

        $result = ['success' => false];

        try {
            $resultCurl = $curl->read();
            $result['response'] = $resultCurl;

            if (!empty($resultCurl)) {
                $result['status'] = Zend_Http_Response::extractCode($resultCurl);
                if (isset($result['status']) && in_array($result['status'], [200, 201])) {
                    $result['success'] = true;
                } else {
                    $result['message'] = __('Cannot connect to server. Please try again later.');
                }
            } else {
                $result['message'] = __('Cannot connect to server. Please try again later.');
            }
        } catch (Exception $e) {
            $result['message'] = $e->getMessage();
        }
        $curl->close();
        return $result;
    }
}
