<?php

namespace Abit\Webhook\Observer;

use Abit\Webhook\Helper\Data;
use Abit\Webhook\Model\HookFactory;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\OrderFactory;
use Magento\Store\Model\StoreManager;

class AfterOrder implements ObserverInterface
{
    /**
     * @var HookFactory
     */
    private $_hookFactory;
    /**
     * @var Data
     */
    private $_helper;

    /**
     * @var OrderRepositoryInterface
     */
    private $_orderRepository;
    /**
     * @var OrderFactory
     */
    private $_orderFactory;
    /**
     * @var StoreManager
     */
    private $_storeManager;
    /**
     * @var \Magento\Sales\Model\ResourceModel\Order
     */
    private $_orderResourceModel;

    public function __construct(
        HookFactory $hookFactory,
        Data $data,
        OrderFactory $orderFactory,
        OrderRepositoryInterface $orderRepository,
        StoreManager $storeManager,
        \Magento\Sales\Model\ResourceModel\Order $orderResourceModel
    ) {
        $this->_hookFactory = $hookFactory;
        $this->_helper = $data;
        $this->_orderFactory = $orderFactory;
        $this->_orderRepository = $orderRepository;
        $this->_orderResourceModel = $orderResourceModel;
        $this->_storeManager = $storeManager;
    }

    /**
     * @inheritDoc
     */
    public function execute(Observer $observer)
    {
        $obj = new \stdClass();
        $orderId= $observer->getEvent()->getOrderIds()[0];
        $orders_items = $this->_orderFactory->create()->load($orderId)->getAllItems();
        $orders =  $this->_orderRepository->get($orderId);
        $orderIncrementId  = $this->_orderRepository->get($orderId)->getIncrementId();
        $orderEntityId = $orders->getEntityId();
        $billinfo= $orders->getBillingAddress()->getData();
        $shippinginfo= $orders->getShippingAddress()->getData();
        $status = $orders->getStatus();
        $shipping_method = $orders->getShippingMethod();
        $coupon_code = $orders->getCouponCode();
        $shipping_total = $orders->getShippingAmount();
        $shipping_tax = $orders->getShippingTaxAmount();
        $sub_total = $orders->getSubtotal();
        $total = $orders->getGrandTotal();
        $total_tax = $orders->getTaxAmount();
        $baseUrl = $this->_storeManager->getStore()->getBaseUrl();
        $statusHistory = $orders->getStatusHistoryCollection();
        $payments = $orders->getPaymentsCollection();
        $payment_method = '';
        foreach ($payments as $payment) {
            $payment_method = $payment->getMethod();
        }
        $message = '';
        foreach ($statusHistory as $status) {
            $message = $status->getComment();
        }

        if (isset($_COOKIE['sourceads'])) {
            $source_ads = $_COOKIE['sourceads'];
        } else {
            $source_ads = Null;
        }
        if (isset($_COOKIE['chiendich'])) {
            $chien_dich = $_COOKIE['chiendich'];
        } else {
            $chien_dich = Null;
        }
        if (isset($_COOKIE['nhomchiendich'])) {
            $nhom_chien_dich = $_COOKIE['nhomchiendich'];
        } else {
            $nhom_chien_dich = Null;
        }
        if (isset($_COOKIE['nhanvien'])) {
            $nhanvien = $_COOKIE['nhanvien'];
        } else {
            $nhanvien = Null;
        }
        if (isset($_COOKIE['chanel'])) {
            $chanel = $_COOKIE['chanel'];
        } else {
            $chanel = Null;
        }
        $data = [
            'sourceads' => $source_ads,
            'chiendich' => $chien_dich,
            'nhomchiendich' => $nhom_chien_dich,
            'nhanvien' => $nhanvien,
            'chanel' => $chanel,
        ];
        $orders->addData($data);
        $this->_orderResourceModel->save($orders);

        $listProduct = [];

        foreach ($orders_items as $key =>  $order) {
            $listProduct[$key]['product_id'] = $order->getProductId();
            $listProduct[$key]['name'] = $order->getName();
            $listProduct[$key]['sku'] = $order->getSku();
            $listProduct[$key]['weight'] = $order->getWeight();
            $listProduct[$key]['price'] = $order->getPrice();
            $listProduct[$key]['qty'] = $order->getQtyOrdered();
            $listProduct[$key]['discount_percent'] = $order->getDiscountPercent();
            $listProduct[$key]['discount_amount'] = $order->getDiscountAmount();
            $listProduct[$key]['tax_percent'] = $order->getTaxPercent();
            $listProduct[$key]['total_tax'] = $order->getTaxAmount();
        }
        $obj->entityId = $orderEntityId;
        $obj->list_pageitem = $baseUrl;
        $obj->shippingMethod = $shipping_method;
        $obj->paymentMethod = $payment_method;
        $obj->billing = $billinfo;
        $obj->shipping = $shippinginfo;
        $obj->orderid = $orderIncrementId;
        $obj->listProduct = $listProduct;
        $obj->status = $status;
        $obj->message = $message;
        $obj->sourceads = $source_ads;
        $obj->chiendich = $chien_dich;
        $obj->nhomchiendich = $nhom_chien_dich;
        $obj->nhanvien = $nhanvien;
        $obj->chanel = $chanel;
        $obj->couponCode = $coupon_code;
        $obj->shippingTotal = $shipping_total;
        $obj->shippingTax = $shipping_tax;
        $obj->subTotal = $sub_total;
        $obj->total = $total;
        $obj->totalTax = $total_tax;

        $obj=json_encode($obj);

        $this->_helper->send($obj);
    }
}
