<?php

namespace Abit\Webhook\Setup;

use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class InstallSchema implements InstallSchemaInterface
{

    /**
     * @inheritDoc
     * @throws \Zend_Db_Exception
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;

        $installer->startSetup();

        $connection = $installer->getConnection();

        if (!$installer->tableExists('abit_webhook_hook')) {
            $table = $connection->newTable($installer->getTable('abit_webhook_hook'))
                ->addColumn(
                    'hook_id',
                    Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'nullable' => false, 'primary' => true, 'unsigned' => true],
                    'Hook Id'
                )
                ->addColumn('name', Table::TYPE_TEXT, 255, ['nullable' => false], 'Name')
                ->addColumn('store_ids', Table::TYPE_TEXT, 64, ['nullable' => false], 'Stores')
                ->addColumn('delivery_url', Table::TYPE_TEXT, 512, ['nullable' => false], 'Delivery Url')
                ->addColumn('webhook_secret', Table::TYPE_TEXT, 64, ['nullable' => false], 'WebHook Secret')
                ->addColumn('webhook_topic', Table::TYPE_TEXT, 64, ['nullable' => false], 'WebHook Topic')
                ->addColumn('method', Table::TYPE_TEXT, 64, [], 'Method')
                ->addColumn('content_type', Table::TYPE_TEXT, 64, ['nullable' => false], 'Content Type')
                ->addColumn('status', Table::TYPE_INTEGER, 1, ['nullable' => false], 'Hook Status')
                ->addColumn(
                    'created_at',
                    Table::TYPE_TIMESTAMP,
                    null,
                    ['default' => Table::TIMESTAMP_INIT],
                    'Created At'
                )
                ->addColumn(
                    'updated_at',
                    Table::TYPE_TIMESTAMP,
                    null,
                    ['default' => Table::TIMESTAMP_INIT],
                    'Update At'
                )
                ->addIndex($installer->getIdxName('abit_webhook_hook', ['hook_id']), 'hook_id')
                ->addIndex($installer->getIdxName('abit_webhook_hook', ['status']), 'status')
                ->setComment('Hook Table');

            $connection->createTable($table);
        }
        if (!$installer->tableExists('abit_webhook_hook_log')) {
            $table = $connection->newTable($installer->getTable('abit_webhook_hook_log'))
                ->addColumn(
                    'id',
                    Table::TYPE_INTEGER,
                    null,
                    ['identity' => true, 'nullable' => false, 'primary' => true, 'unsigned' => true],
                    'Log Id'
                )
                ->addColumn(
                    'hook_id',
                    Table::TYPE_INTEGER,
                    null,
                    ['nullable' => false, 'unsigned' => true],
                    'Hook Id'
                )
                ->addIndex($installer->getIdxName('abit_webhook_hook_log', ['hook_id']), ['hook_id'])
                ->addForeignKey(
                    $installer->getFkName('abit_webhook_hook_log', 'hook_id', 'abit_webhook_hook', 'hook_id'),
                    'hook_id',
                    $installer->getTable('abit_webhook_hook'),
                    'hook_id',
                    Table::ACTION_CASCADE
                )
                ->addColumn('status', Table::TYPE_TEXT, 64, [], 'Log Status')
                ->addColumn('response', Table::TYPE_TEXT, 255, [], 'Response')
                ->addColumn('delivery_url', Table::TYPE_TEXT, 255, [], 'Delivery Url')
                ->addColumn('message', Table::TYPE_TEXT, 255, [], 'Message')
                ->addColumn('body', Table::TYPE_TEXT, 255, [], 'Body')
                ->addColumn(
                    'created_at',
                    Table::TYPE_TIMESTAMP,
                    null,
                    ['default' => Table::TIMESTAMP_INIT],
                    'Created At'
                )
                ->addColumn(
                    'updated_at',
                    Table::TYPE_TIMESTAMP,
                    null,
                    ['default' => Table::TIMESTAMP_INIT],
                    'Update At'
                )
                ->setComment('Hook Log Table');

            $connection->createTable($table);
        }
        $installer->endSetup();
        // TODO: Implement install() method.
    }
}
